/*
 * JMapPanel.java
 *
 * Created on 25. �erven 2007, 23:00
 */

package pathfinder;

import gui.MapFrame;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Line2D;
import java.util.List;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;

/** JMapPanel - a component utilised for viewing the m_map.
 * No editing is expected to be done here
 * @author  Petr Buryan
 */
public class JMapPanel extends javax.swing.JPanel {
    protected int fWidth,fHeight;
    protected int m_cityWidth=10;
    protected int m_cityHeight=10;
    protected Color highwayColor=Color.RED;
    protected Color firstColor=Color.black;
    protected Color secondColor=Color.gray;
    protected Color pathColor=Color.blue;
    protected Color cityColor=Color.orange;
    protected JPFMap m_map=null;
    protected MapFrame m_parentFrame;

    private double m_margin=15d;
    private JPopupMenu m_popupMenu;
    private float m_minClickDistance=15f;
    
    /** Creates new form JMapPanel */
    public JMapPanel() {
        initialise();
    }
    
    public JMapPanel(int width, int height, JPFMap m) {
        super();
        initialise();
        fWidth = width;
        fHeight= height;
        m_map=m;
    }
    
    public JMapPanel(int width, int height, List Cts, List Rds) {
        super();
        initialise();
        fWidth = width;
        fHeight= height;
    }

    public void initialise(){
        initComponents();
        final JMapPanel panel = this;
        m_popupMenu = new JPopupMenu();

        // Create and add a menu item
        final JMenuItem item = new JMenuItem("Select as start");
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String[] coords=m_popupMenu.getName().split(":");
                int city=getNearestCity(Float.parseFloat(coords[0]), Float.parseFloat(coords[1]));
                //System.out.println("Start set to "+city);
                m_parentFrame.setStartCity(city);
                m_map.highlightCities(new int[]{city});
            }
        });
        m_popupMenu.add(item);
        // Create and add a menu item
        JMenuItem item2 = new JMenuItem("Select as finish");
        item2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String[] coords=m_popupMenu.getName().split(":");
                int city=getNearestCity(Float.parseFloat(coords[0]), Float.parseFloat(coords[1]));
                //System.out.println("Finish set to "+city);
                m_map.highlightCities(new int[]{city});
                m_parentFrame.setStopCity(city);
            }
        });
        m_popupMenu.add(item2);

        m_popupMenu.add(new JSeparator());

        // Create and add a menu item
        JMenuItem item3 = new JMenuItem("Block road");
        item3.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String[] coords=m_popupMenu.getName().split(":");
                int[] road=getNearestRoad(Float.parseFloat(coords[0]), Float.parseFloat(coords[1]));
                if(road==null)
                    return;
                System.out.println("Blocking road "+road[0]+"-"+road[1]);
                m_map.blockRoad(road[2]);
                EventQueue.invokeLater(new Runnable() {
                    public void run() {
                        panel.repaint();
                    }
                });
                //panel.repaint();
            }
        });
        m_popupMenu.add(item3);

        // Set the component to show the popup menu
        this.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent evt) {
                if(m_parentFrame==null)
                    return;
                if (evt.isPopupTrigger()) {
                    //dirty trick to quickly save the click coordinates
                    m_popupMenu.setName(evt.getX()+":"+evt.getY());
                    m_popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
                }
            }
            @Override
            public void mouseReleased(MouseEvent evt) {
                if(m_parentFrame==null)
                    return;
                if (evt.isPopupTrigger()) {
                    //dirty trick to quickly save the click coordinates
                    m_popupMenu.setName(evt.getX()+":"+evt.getY());
                    m_popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
                }
            }
        });

    }

    public void setParentFrame(MapFrame parentFrame) {
        this.m_parentFrame = parentFrame;
    }

    public void setCityWidth(int w){
        m_cityWidth=w;
    }
    
    public void setCityHeight(int h){
        m_cityHeight=h;
    }
    
    public void setRoadColors(Color Hi, Color Mid, Color Lo){
        highwayColor=Hi;
        firstColor=Mid;
        secondColor=Lo;
    }
    
    public void setPathColor(Color P){
        pathColor=P;
    }
    
    public void setCityColor(Color C){
        cityColor=C;
    }
    
    public int[] getCity(int ci){
        int[] c=new int[2];
        for(int i=0;i<m_map.getCityList().size(); i++){
            String[] s=(String[]) m_map.getCityList().get(i);
            int id = Integer.valueOf(s[0]);
            if(id==ci){
                c[0] = Integer.valueOf((int) Math.round(Double.valueOf(s[1])));
                c[1] = Integer.valueOf((int) Math.round(Double.valueOf(s[2])));
                return c;
            }
        }
        return null;
    }

    /** Returns id of the city that is nearest to given position
     * @param x x coordinate
     * @param y y coordinate
     * @return id of the nearest city or -1 if there is no such city
     */
    public int getNearestCity(float x, float y){
        Dimension d=this.getSize();
        double w=d.getWidth()-this.getBounds().x;
        double h=d.getHeight()-this.getBounds().y;
        double[] cc= {x, y};
        int[]icc =m_map.remapCoordsBack(cc,w,h);
        return m_map.getNearestCity(icc[0], icc[1]);
        //m_map.remapCoords(new int[]{75,70}, w, h)[1]
    }

    /** Returns id of the cities that are connected by road that is nearest to given position
     * @param x x coordinate
     * @param y y coordinate
     * @return id of the road cities or null if there is no such city
     */
    public int[] getNearestRoad(float x, float y){
        Dimension d=this.getSize();
        double w=d.getWidth()-this.getBounds().x;
        double h=d.getHeight()-this.getBounds().y;
        double[] cc= {x, y};
        int[]icc =m_map.remapCoordsBack(cc,w,h);
        return m_map.getNearestRoad(icc[0], icc[1], m_minClickDistance);
        //m_map.remapCoords(new int[]{75,70}, w, h)[1]
    }

    public void drawCity(Graphics g, int x, int y, String label){
        Graphics2D g2d = (Graphics2D)g;
        Dimension d=this.getSize();
        double w=d.getWidth()-m_margin;
        double h=d.getHeight()-m_margin;
        double[] cc= {x, y};
        int[]icc =m_map.remapCoords(cc,w,h);

        float dash[] = {1,0};
        BasicStroke bs = new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 10, dash, 0);
        g2d.setStroke(bs);
        
        g2d.setPaint(Color.white);
        g2d.fillOval(icc[0]+m_cityWidth/3,icc[1]-m_cityHeight/2, 2*m_cityWidth, 2*m_cityHeight);
        g2d.setPaint(Color.black);
        g2d.drawString(label,icc[0]+m_cityWidth,icc[1]+m_cityHeight);
        
        g2d.setPaint(cityColor);
        g2d.fillOval(icc[0]-m_cityWidth/2, icc[1]-m_cityWidth/2, m_cityWidth+1, m_cityHeight+1);
        
        g2d.setPaint(Color.black);
        g2d.drawOval(icc[0]-m_cityWidth/2, icc[1]-m_cityWidth/2, m_cityWidth, m_cityHeight);
        g2d.drawOval(icc[0]-m_cityWidth/2+2, icc[1]-m_cityWidth/2+2, m_cityWidth-4, m_cityHeight-4);
    }

    

    public void drawCity(int city){
        Graphics g=getGraphics();
        if (g==null){return;}
        Graphics2D g2d = (Graphics2D) g;
        int[] city1=getCity(city);
        int x=city1[0];
        int y=city1[1];
        Dimension d=this.getSize();
        double w=d.getWidth()-m_margin;
        double h=d.getHeight()-m_margin;
        double[] cc= {x, y};
        int[]icc =m_map.remapCoords(cc,w,h);

        g2d.setPaint(cityColor);
        g2d.fillOval(icc[0]-m_cityWidth/2, icc[1]-m_cityWidth/2, m_cityWidth+1, m_cityHeight+1);
        g2d.setPaint(Color.BLACK);
        g2d.drawOval(icc[0]-m_cityWidth/2, icc[1]-m_cityWidth/2, m_cityWidth, m_cityHeight);
        g2d.drawOval(icc[0]-m_cityWidth/2+2, icc[1]-m_cityWidth/2+2, m_cityWidth-4, m_cityHeight-4);
    }
    
    public void drawRoad(Graphics g, int c1, int c2, int spd){
        Graphics2D g2d = (Graphics2D)g;
        Shape road;
        int[] city1=getCity(c1);
        int[] city2=getCity(c2);
        if((city1==null)||(city2==null)){
            return;
        }
        
        Dimension d=this.getSize();
        double w=d.getWidth()-m_margin;
        double h=d.getHeight()-m_margin;
        
        city1 =m_map.remapCoords(city1, w,h);
        city2 =m_map.remapCoords(city2, w,h);
        int c1x=city1[0];
        int c1y=city1[1];
        int c2x=city2[0];
        int c2y=city2[1];
        
        g2d.setPaint(highwayColor);
        road = new Line2D.Float(c1x,c1y,c2x,c2y);
        
        if (spd==90){
            g2d.setPaint(firstColor);
            road=new CubicCurve2D.Double(c1x,c1y,c1x+0.8*m_cityWidth,c1y,c2x-1.2*m_cityWidth,c2y,c2x,c2y); //(Bezi�rova k�ivka)
        }
        if (spd==60){
            g2d.setPaint(secondColor);
            road=new CubicCurve2D.Double(c1x,c1y,c1x+2.2*m_cityHeight,c1y,c2x-2.7*m_cityWidth,c2y,c2x,c2y); //(Bezi�rova k�ivka)
        }
        float dash[] = {5,4};
        BasicStroke bs = new BasicStroke(3f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 10, dash, 0);
        if (spd==90){
            dash = new float[]{10,4};
            bs = new BasicStroke(2f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 10, dash, 0);
        }
        else if (spd==60){
            dash = new float[]{15,4};
            bs = new BasicStroke(1.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 10, dash, 0);
        }
        g2d.setStroke(bs);
        g2d.draw(road);
        
    }
    
    public void drawMap(Graphics g){
        for(int i=0;i<m_map.getRoadList().size(); i++){
            String[] s=(String[]) m_map.getRoadList().get(i);
            int c1=Integer.valueOf(s[0]);
            int c2=Integer.valueOf(s[1]);
            int spd=Integer.parseInt(s[2]);
            if(spd<=0)
                continue;
            drawRoad(g,c1,c2,spd);
        }
        
        for(int i=0;i<m_map.getCityList().size(); i++){
            String[] s=(String[]) m_map.getCityList().get(i);
            int x=Integer.valueOf((int) Math.round(Double.valueOf(s[1])));
            int y=Integer.valueOf((int) Math.round(Double.valueOf(s[2])));
            String label=s[0];
            if (s.length>4) {
                int pop=Integer.parseInt(s[4]);
                if (pop>1e4) label=s[3];
            }
            drawCity(g,x,y,label);
        }
    }
    
    public void drawPath(int c1, int c2){
        drawPath(c1, c2, pathColor);
    }
    
    public void drawPath(int c1, int c2, Color col){
        Graphics g=getGraphics();
        if (g==null){return;}
        Graphics2D g2d = (Graphics2D) g;
        
        Shape road;
        int[] city1=getCity(c1);
        int[] city2=getCity(c2);
        Dimension d=this.getSize();
        double w=d.getWidth()-m_margin;
        double h=d.getHeight()-m_margin;
        city1 =m_map.remapCoords(city1,w,h);
        city2 =m_map.remapCoords(city2,w,h);
        
        g2d.setPaint(col);
        road = new Line2D.Float(city1[0],city1[1],city2[0],city2[1]);
        float dash[] = {5,0};
        BasicStroke bs = new BasicStroke(3, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 10, dash, 0);
        g2d.setStroke(bs);
        g2d.draw(road);
    }
    
    @Override
    public void paintComponent(Graphics g) {
        // First paint background unless you will paint whole area yourself.
        setBackground(Color.white);
        super.paintComponent(g);
        drawMap(g);
    } // paintComponent
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    
}
